@@ -151,14 +151,10 @@ knit = function(
       useFancyQuotes = FALSE, device = pdf_null, knitr.in.progress = TRUE
     )
     on.exit(options(oopts), add = TRUE)
-    # restore chunk options after parent exits
-    optc = opts_chunk$get(); ocode = knit_code$get(); optk = opts_knit$get()
-    on.exit({
-      opts_chunk$restore(optc)
-      knit_code$restore(ocode)
-      opts_current$restore()
-      opts_knit$restore(optk)
-    }, add = TRUE)
+    # restore objects like chunk options after parent exits
+    opta = list(opts_chunk, opts_current, knit_code, opts_knit)
+    optv = lapply(opta, function(o) o$get())
+    on.exit(for (i in seq_along(opta)) opta[[i]]$restore(optv[[i]]), add = TRUE)
     opts_knit$set(
       output.dir = getwd(),  # record working directory in 1st run
       tangle = tangle, progress = opts_knit$get('progress') && !quiet
@@ -311,20 +307,18 @@ process_file = function(text, output) {
     }
     if (progress && is.function(pb$update)) pb$update(i)
     group = groups[[i]]
-    res[i] = withCallingHandlers(
+    knit_concord$set(block = i)
+    res[i] = xfun:::handle_error(
       withCallingHandlers(
         if (tangle) process_tangle(group) else process_group(group),
         error = function(e) if (xfun::pkg_available('rlang', '1.0.0')) rlang::entrace(e)
       ),
-      error = function(e) {
+      function(e, loc) {
         setwd(wd)
         write_utf8(res, output %n% stdout())
-        message(
-          '\nQuitting from lines ', paste(current_lines(i), collapse = '-'),
-          if (labels[i] != '') sprintf(' [%s]', labels[i]),
-          sprintf(' (%s)', knit_concord$get('infile'))
-        )
-      }
+        paste0('\nQuitting from lines ', loc)
+      },
+      if (labels[i] != '') sprintf(' [%s]', labels[i]), get_loc
     )
   }
 
@@ -337,6 +331,11 @@ process_file = function(text, output) {
   res
 }
 
+# return a string to point out the current location in the doc
+get_loc = function(label = '') {
+  paste0(current_lines(), label, sprintf(' (%s)', knit_concord$get('infile')))
+}
+
 auto_out_name = function(input, ext = tolower(file_ext(input))) {
   base = sans_ext(input)
   name = if (opts_knit$get('tangle')) c(base, '.R') else
