@@ -64,13 +64,14 @@ color_def = function(col, variable = 'shadecolor') {
       x = switch(variable, shadecolor = rep(.97, 3), fgcolor = rep(0, 3))
       warning("the color '", col, "' is invalid;",
               'using default color...',
-              'see https://yihui.org/knitr/options')
+              'see https://yihui.org/knitr/options/')
     }
   }
   if (length(x) != 3L) stop('invalid color:', col)
   if (is.numeric(x)) x = round(x, 3L)
-  outdec = options(OutDec = '.'); on.exit(options(outdec))
-  sprintf('\\definecolor{%s}{rgb}{%s, %s, %s}', variable, x[1], x[2], x[3])
+  xfun::decimal_dot(
+    sprintf('\\definecolor{%s}{rgb}{%s, %s, %s}', variable, x[1], x[2], x[3])
+  )
 }
 
 # split by semicolon or colon
@@ -204,7 +205,6 @@ format_sci_one = function(
     return(round_digits(x)) # no need sci notation
 
   b = round_digits(x / 10^lx)
-  b[b %in% c(1, -1)] = ''
 
   switch(format, latex = {
     sci_notation('%s%s10^{%s}', b, times, lx)
@@ -225,7 +225,11 @@ format_sci_one = function(
 }
 
 sci_notation = function(format, base, times, power) {
-  sprintf(format, base, ifelse(base == '', '', times), power)
+  if (base %in% c('1', '-1')) {
+    times = ''
+    base = if (base == '1') '' else '-'
+  }
+  sprintf(format, base, times, power)
 }
 
 # vectorized version of format_sci_one()
@@ -243,19 +247,18 @@ tikz_dict = function(path) {
   paste(sans_ext(basename(path)), 'tikzDictionary', sep = '-')
 }
 
-# convert dashes in option names with dots (e.g., `fig-height` to `fig.height`)
-dot_names = function(x) {
-  dashes = grep('-', names(x), value = TRUE)
-  dots   = gsub('-', '.', dashes)
+# convert dashes in option names with dots (e.g., `fig-height` to `fig.height`),
+# and vice versa (dots work for knitr, and dashes for Quarto)
+fix_names = function(x, char1, char2, aliases) {
+  name1 = grep(char1, names(x), value = TRUE, fixed = TRUE)
+  name2 = gsub(char1, char2, name1, fixed = TRUE)
+  name3 = c(names(opts_chunk_attr), names(opts_chunk$get()))
   # only convert names that are known to knitr
-  i = dots %in% c(names(opts_chunk_attr), names(opts_chunk$get()))
+  i = (if (char1 == '.') name1 else name2) %in% name3
   if (any(i)) {
-    x[dots[i]] = x[dashes[i]]
-    x[dashes[i]] = NULL
+    x[name2[i]] = x[name1[i]]  # move values to to new names
+    x[name1[i]] = NULL  # delete old names
   }
-
-  # normalize aliases (introduced by Quarto)
-  aliases = c(fig.format = 'dev', fig.dpi = 'dpi')
   for (j in intersect(names(x), names(aliases))) {
     x[[aliases[j]]] = x[[j]]
     x[[j]] = NULL
@@ -263,6 +266,14 @@ dot_names = function(x) {
   x
 }
 
+dot_names = function(x) {
+  fix_names(x, '-', '.', c(fig.format = 'dev', fig.dpi = 'dpi'))
+}
+
+dash_names = function(x) {
+  fix_names(x, '.', '-', c(dev = 'fig-format', dpi = 'fig-dpi', tab.cap = 'tbl-cap'))
+}
+
 # initially for compatibility with Sweave and old beta versions of knitr
 # but now also place to tweak default options
 fix_options = function(options) {
@@ -1041,8 +1052,14 @@ raw_output = function(x, markers = raw_markers, ...) {
 #' knitr::raw_latex('\\emph{some text}')
 raw_block = function(x, type = 'latex', ...) {
   if (!rmarkdown::pandoc_available('2.0.0')) warning('raw_block() requires Pandoc >= 2.0.0')
-  x = c(sprintf('\n```{=%s}', type), x, '```\n')
-  asis_output(one_string(x), ...)
+  x = fenced_block(x, attr = paste0('=', type))
+  x = gsub('^\n|\n$', '', x)
+  # TODO: get rid of this hack for davidgohel/flextable#621
+  if ('flextable' %in% xfun:::sys.packages() && packageVersion('flextable') <= '0.9.5') {
+    x = gsub('^(\\s*)```+', '\\1```', x)
+    x = gsub('```+(\\s*)$', '```\\1', x)
+  }
+  asis_output(x, ...)
 }
 
 #' @rdname raw_block
@@ -1131,9 +1148,7 @@ str_split = function(x, split, ...) {
 # methods to update/close it
 txt_pb = function(total, labels) {
   if (getOption('knitr.progress.linenums', FALSE)) {
-    nums = sapply(seq_along(labels), function(i) {
-      paste(current_lines(i), collapse = '-')
-    })
+    nums = sapply(seq_along(labels), current_lines)
     labels = sprintf(
       '%s%s%s:%s', labels, ifelse(labels == '', '', ' @ '),
       knit_concord$get('infile'), nums
@@ -1155,10 +1170,8 @@ txt_pb = function(total, labels) {
     if (identical(con, '')) con = stdout()
     if (!inherits(con, 'connection')) return(TRUE)
     if (isatty(con)) return(FALSE)
-    # RStudio's background jobs
-    if (Sys.getenv('RSTUDIO_CHILD_PROCESS_PANE') %in% c('job', 'build')) return(FALSE)
-    # when RStudio is available, return FALSE
-    is.null(tryCatch(rstudioapi::versionInfo(), error = function(e) NULL))
+    # when in RStudio, return FALSE
+    Sys.getenv('RSTUDIO') != '1'
   })()
   # use simple progress output without the bar but only progress and labels
   if (simple) return(list(
